def fn check_simple_move% (square%,direction%)
rem This returns bitmap of moves possible on the square given
rem 1 = left
rem 2 = right
rem 4 = up
rem 8 = down
rem Square% is the square we just moved to and where we must eval the
rem possible moves.
rem Direction% is the direction in which we moved to get to this square.
rem We consider that, on this square, the opposite direction isn't
rem possible (else we could pingpong back & forth).
rem So, if we moved left (1) to get to this square, then right (2) shouldn't
rem be a possible move.
rem This also makes the "preferred direction", which is the direction
rem we should try to move to get to our goal.
local res%,row%,col%,temp%
  res%=0				: rem cumulative result
  if square%<1:ret 0			: rem no moves here at all
  if square%>stopsquare%:ret 0		: rem nor here
:
rem check left	(decrease by 1)
:
  if ((square% mod rows%)<>1 ) and (tried%(square%,0)=0)
    if direction%<>2
      if squares%(square%-1)=0:res%=1	: rem not on the left side of board...
    endif
  endif 				: rem  & square to the left is unoccupied
rem check right (increase by 1)
  if ((square% mod rows%)<>0 ) and (tried%(square%,1)=0)
    if direction%<>1
      if squares%(square%+1)=0:res%=res%+2: rem not on right hand side & right empty
    endif
  endif
rem check up  (decrease by rows%)
  if (square%>rows%) and (tried%(square%,2)=0)
    if direction%<>8
      if squares%(square%-rows%)=0:res%=res%+4
    endif
  endif
rem check down (increase by rows%)
  if (square%<=stopsquare%-rows%) and (tried%(square%,3)=0)
    if direction%<>4
      if squares%(square%+rows%)=0:res%=res%+8
    endif
  endif
rem now make the preferred directions, uses the global vars sqcol% and sqrow%
rem which are the col & row of the "goal" square
rem the preference is for left or right first, then up or down
rem unless we're in the same column, the up or down first
  row%=(square%-1 div rows%)+1		: rem square row
  col%=square% mod rows%		: rem square column
  if col%=0:col%=rows%
  if col%>sqcol%
    prefer%(0)=1			: rem preferred move : left
    prefer%(2)=2
  else
    prefer%(0)=2			: rem preferred move : right
    prefer%(2)=1
  endif
  if row%>sqrow%
    prefer%(1)=4			: rem same for up/down
    prefer%(3)=8
  else
    prefer%(1)=8
    prefer%(3)=4
  endif
  if col%=sqcol%
    temp%=prefer%(0)			: rem if columns are same, prefered moves = up/down!
    prefer%(0)=prefer%(1)
    prefer%(1)=temp%
    temp%=prefer%(2)
    prefer%(2)=prefer%(3)
    prefer%(3)=temp%
  endif
  ret res%
end def check_simple_move%
:
def fn check_possible (start%,goal%)
rem This checks whether it is possible to move a stone from square start% to
rem square goal%.
rem way%(rows%*2) are the possible squares one can pass through make and
rem it contains the way followed by the "man" up to now.
rem This function returns 1 if move possible, 0 if not
local current_way%,way%(stopsquare%), tried%(rows%*rows%,3)
  if start%=goal%:return 0		: rem what?
  current_way%=0			: rem what we did up to now
  current%=start%			: rem this is where we start checking
  ret%=check_this_square%(start%,0)   : rem this recursive function does all the work
  ret ret%				: rem OK or not
end def check_possible
:
def fn check_this_square%(current%,direction%)
rem This checks the square given as parameter
rem direction% is the one we came from: no need to check that
rem i.e. if we went left, we come from the right of the current% square
rem Is it the goal? If yes return 1
rem   If not, have we been here already in this way? yes- go back
rem	If not, can we move?
rem	 If yes, can we move into the preferred direction(s)?
rem	   yes, move into preferred direction
rem	   no, move into another direction
rem    no, we can't move, return 0, square is impossible
rem (default preferred directions are : 1-left 2 - up 3-right 4-down)
rem returns 1 if we reached goal, else 0
local possibles%,prefer%(3),temp%,lp%,temp2%
rem  report_in current%,direction%,current_way%
rem  if squares%(current%)<>0:print #kk%; "NOT EMPTY!!!!!"
  if current%=goal% : return 1		: rem gotcha
  if check_previous%(current%):return 0 : rem we're coming back to a square!
  current_way%=current_way%+1		: rem next current way
  way%(current_way%)=current%		: rem show what we did up to now
  possibles%=check_simple_move%(current%,direction%) : rem get possible moves
  for lp%=0 to 3
    temp%=prefer%(lp%)			: rem get first preferred dir & try to go there
    if possibles%&&temp%
      select on temp%
	=1:temp%=-1:temp2%=0
	=2:temp%=1:temp2%=1
	=4:temp%=-rows%:temp2%=2
	=8:temp%=rows% :temp2%=3
      end select
      tried%(current%,temp2%)=1 	: rem show that we tried this already
      if check_this_square%(current%+temp%,prefer%(lp%)):return 1: rem check next square!
    end if
  end for lp%
  rem if we come here, none of the 4 moves were a positive result
  current_way%=current_way%-1		: rem this square was no good!
  return 0				: rem show it
end def check_this_square%
:
def fn check_previous% (x%)
rem this checks that we haven't come to a previous square by going in a circle
rem if yes, we must scratch it
rem returns 1 if a square alreay visited, else 0
local lp%
  if current_way%<1:return 0
  for lp%=1 to current_way%
    if way%(lp%)=x%:return 1
  end for lp%
  return 0
end def check_previous
:
